
### Example Python script to calculate the discordance score ###

""" It requires input expression data in text file format. 
    repressive tendency score files (repressive_hg19.txt and repressive_mm10.txt) MUST be also in the same directory.

    Input file is a text file composed of two columns - the first column shows gene names, the second column shows the expression value.
    For example, See example_input.txt 

    Parameters
     -i input file name (required)
     -o output file name (required)
     -p pesudo-count to be added to the input data, default = 0
     -l log-transformation of the input data, default = False
     -s species, hg19 or mm10 (required)
     -f minimum input value allowed, default = 1 

     E.g. >>> python discordance.py -i example_input.txt -o output_file.txt -s hg19 -l False 
        Run the analysis with the human data (hg19) without the log-transformation of the input data """


import numpy as np
import sys
from optparse import OptionParser


def read_file(filename):    
    results = {}
    file_ = open(filename, 'r')
    for line in file_:
        line = line.strip().split()
        results[line[0]] = float(line[1])
    return results


def perform_analysis(exp, ref, pseudo_=0, exp_filter=1.0, fill_missing=True, log_conversion=False):     
    if pseudo_!=None:
        exp = add_pseudo(exp, count_=pseudo_)
    if log_conversion==True:
        for g in exp:
            exp[g] = np.log10(exp[g])
    results = calculate_scores(exp, ref, exp_filter=exp_filter, fill_missing=True)
    results = sort_(results, idx=1, reverse_=True)    
    results.insert(0, ['\t','score'])    
    return results


def sort_(input_list, idx, reverse_ = False, numerical=True):    
    if numerical==True:
        for no in range(len(input_list)):
            input_list[no][idx] = float(input_list[no][idx])
    return sorted(input_list, key=lambda x: x[idx], reverse=reverse_)


def calculate_scores(dic1, dic2, exp_filter=1.0, fill_missing=True):    
    all_genes = []
    min_ = 1.0
    for g in dic1:        
        all_genes.append(g)    
    for g in dic2:
        if float(dic2[g]) < min_:
            min_ = float(dic2[g])   
    if fill_missing==True:
        for g in all_genes:
            if g not in dic2:
                dic2[g] = min_
    results = []    
    for gene in all_genes:
        if dic1[gene] < exp_filter:
            value = 0.0
        else:            
            value = dic1[gene] * dic2[gene]            
        results.append([gene, value])
    return results


def add_pseudo(input_data, count_):
    for gene_ in input_data:        
        input_data[gene_] += count_
    return input_data


def write_file(input_data, output_file):
    output_ = open(output_file, 'w')
    for i in input_data:        
        output_.write(str(i[0])+'\t'+str(i[1])+'\n')
    output_.close()

if __name__ == '__main__':

    # Command line options    
    parser = OptionParser()
    parser.add_option('-i', '--i', dest='input_file', help='input filename')    
    parser.add_option('-o','--o', dest='output_file', help='output filename')
    parser.add_option('-p', '--p', dest='pseudo', help='pesudo count', default=None)
    parser.add_option('-l', '--l', dest='log_transform', help='log-transformation (True or False)', default=False)
    parser.add_option('-s', '--s', dest='species', help='human (hg19) or mouse (mm10)')
    parser.add_option('-f', '--f', dest='min_exp', help='minimum expression value allowed', default=1.0)

    options = parser.parse_args()[0]  

    if options.log_transform == 'True':
        options.log_transform = True
    else:
        options.log_transform = False    

    if (options.input_file == None) or (options.output_file == None) or (options.species == None):
        sys.exit(' Input parameters are missing. Exiting. \n At least input filename (-i), output filename (-o) and species (-s) are required.')
    else:
        if options.species == 'hg19':
            ref_ = 'repressive_hg19.txt'
        elif options.species == 'mm10':
            ref_ = 'repressive_mm10.txt'
        
        exp = read_file(options.input_file)
        ref = read_file(ref_)
        results = perform_analysis(exp, ref, pseudo_=options.pseudo, exp_filter=options.min_exp, fill_missing=True, log_conversion=options.log_transform)
        write_file(results, options.output_file)
   


